#encoding = utf8
import logging

from asr import SherpaNcnnAsr
from human.audio_inference_handler import AudioInferenceHandler
from human.audio_mal_handler import AudioMalHandler
from human.human_render import HumanRender
from nlp import PunctuationSplit, DouBao
from tts import TTSEdge, TTSAudioSplitHandle
from utils import load_avatar, get_device

logger = logging.getLogger(__name__)


class HumanContext:
    def __init__(self):
        self._fps = 50  # 20 ms per frame
        self._image_size = 96
        self._batch_size = 16
        self._sample_rate = 16000
        self._stride_left_size = 10
        self._stride_right_size = 10

        self._device = get_device()
        print(f'device:{self._device}')
        full_images, face_frames, coord_frames = load_avatar(r'./face/', self._image_size, self._device)
        self._frame_list_cycle = full_images
        self._face_list_cycle = face_frames
        self._coord_list_cycle = coord_frames
        face_images_length = len(self._face_list_cycle)
        logging.info(f'face images length: {face_images_length}')
        print(f'face images length: {face_images_length}')

        self.asr = None
        self.nlp = None
        self.tts = None
        self.tts_handle = None
        self.mal_handler = None
        self.infer_handler = None
        self._render_handler = None

    @property
    def fps(self):
        return self._fps

    @property
    def image_size(self):
        return self._image_size

    @property
    def device(self):
        return self._device

    @property
    def batch_size(self):
        return self._batch_size

    @property
    def sample_rate(self):
        return self._sample_rate

    @property
    def stride_left_size(self):
        return self._stride_left_size

    @property
    def stride_right_size(self):
        return self._stride_right_size

    @property
    def face_list_cycle(self):
        return self._face_list_cycle

    @property
    def frame_list_cycle(self):
        return self._frame_list_cycle

    @property
    def coord_list_cycle(self):
        return self._coord_list_cycle

    @property
    def render_handler(self):
        return self.render_handler

    def build(self):
        self._render_handler = HumanRender(self, None)
        self.infer_handler = AudioInferenceHandler(self, self._render_handler)
        self.mal_handler = AudioMalHandler(self,  self.infer_handler)
        self.tts_handle = TTSAudioSplitHandle(self,  self.mal_handler)
        self.tts = TTSEdge(self.tts_handle)
        split = PunctuationSplit()
        nlp = DouBao(split,  self.tts)
        self.asr = SherpaNcnnAsr()
        self.asr.attach(nlp)


